{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Parameter Manager"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "class ParameterManager(object):\n",
      "    def __init__(self, \n",
      "                name_manager: NameManager = None, \n",
      "                logger: Logger = None):\n",
      "                    self.setup_name_manager(var=name_manager)\n",
      "                    self.setup_logger(var=logger)\n",
      "                    self._parameters = {}\n",
      "\n",
      "    def setup_name_manager(self, var = None):\n",
      "        if var == None:\n",
      "            try:\n",
      "                self.name_manager = self.logger.name_manager\n",
      "            except:\n",
      "                self.name_manager = NameManager(name=rospy.get_name())\n",
      "        else:\n",
      "            self.name_manager = var\n",
      "            \n",
      "    def setup_logger(self, var = None):\n",
      "        # If we are not given a logger, we will create one using our \n",
      "        # Name manager. Note that if the Name Manager was created\n",
      "        # automatically it already has a '.' in its name, which \n",
      "        # indicates this is a parameter.\n",
      "        if var == None:\n",
      "            self.logger = Logger(name_manager=self.name_manager)\n",
      "        else:\n",
      "            self.logger = var\n",
      "\n",
      "    def add(self, var):\n",
      "        variable = var.name\n",
      "        self.logger.i(f\"Parameter name is {variable}\")\n",
      "        self._parameters[variable] = var\n",
      "        setattr(self, variable, self._parameters[variable].value)\n",
      "        self.logger.i(f\"Parameter {variable} added.\")\n",
      "\n",
      "    def update(self):\n",
      "        count = 0\n",
      "        for parameter in self._parameters:\n",
      "            if self._parameters[parameter].dynamic:\n",
      "                old = self._parameters[parameter]\n",
      "                self._parameters[parameter].refresh_parameter()\n",
      "                new = self._parameters[parameter]\n",
      "                if old != new:\n",
      "                    self.logger().i(\n",
      "                        f'Parameter {parameter} updated to new values.')\n",
      "                    count = count + 1\n",
      "        self.logger.i(f'Parameters updated. {count} updates found.')\n",
      "\n"
     ]
    }
   ],
   "source": [
    "import inspect\n",
    "from viper_toolkit import Dissect\n",
    "from viper_toolkit import ParameterManager, Parameter\n",
    "lines = inspect.getsource(ParameterManager)\n",
    "print(lines)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "scrolled": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "class Parameter(object):\n",
      "    ''' \n",
      "    The parameter object gets, sets, and syncs a local and global param.\n",
      "    \n",
      "    Keyword arguments:\n",
      "    name -- string indicating the name of the variable\n",
      "    target -- string indicating the target address of the parameter, \n",
      "        generally in the form of '/node/name'\n",
      "    default -- the default value, or starting value, for this parameter.\n",
      "    dynamic -- bool indicating if the parameter should be updated automatically\n",
      "    name_manager -- a NameManager object or pointer\n",
      "    logger -- a Logger Object or pointer\n",
      "    '''\n",
      "    \n",
      "    def __init__(self, \n",
      "                name: str = None, \n",
      "                target: str = None,\n",
      "                dynamic: bool = False, \n",
      "                default: any = None,\n",
      "                name_manager: NameManager = None,\n",
      "                #parameters = None,\n",
      "                logger: Logger = None,\n",
      "                value: any = None):\n",
      "                    self.setup_name_manager(var=name_manager)\n",
      "                    self.setup_logger(var=logger)\n",
      "                    self._name = name\n",
      "                    self._target = target\n",
      "                    self._default = default\n",
      "                    self._dynamic = dynamic,\n",
      "                    self.value = value\n",
      "\n",
      "    def setup_name_manager(self, var=None):\n",
      "        if var == None:\n",
      "            self.name_manager = NameManager()\n",
      "        else:\n",
      "            self.name_manager = var\n",
      "            \n",
      "    def setup_logger(self, var=None):\n",
      "        if var == None:\n",
      "            self.logger = Logger(name_manager = self.name_manager)\n",
      "        else:\n",
      "            self.logger = var\n",
      "\n",
      "    @property\n",
      "    def name(self):\n",
      "        return self._name\n",
      "    @name.setter\n",
      "    def name(self, var = None):\n",
      "        if var == None:\n",
      "            try:\n",
      "                self._name = str(self.target.split('/')[-1])\n",
      "                self.logger.i(f\"Parameter name from target is {self._name}\")\n",
      "            except (NameError):\n",
      "                self.logger.e(\"Target not defined.\")\n",
      "                self._name = self.name_manager.name\n",
      "        else:\n",
      "            self._name = str(var)\n",
      "            self.logger.i(f'Passed Parameter name is {self._name}')\n",
      "    \n",
      "    @property\n",
      "    def target(self):\n",
      "        if self._target == None:\n",
      "            self.target = None\n",
      "        return self._target\n",
      "    @target.setter\n",
      "    def target(self, var=None):\n",
      "        if var == None:\n",
      "            try:\n",
      "                my_location = self.name_manager.name\n",
      "            except (NameError):\n",
      "                self.logger.e(\"Name Manager not setup\")\n",
      "                try:\n",
      "                    my_location = rospy.get_name()\n",
      "                except (NameError):\n",
      "                    self.logger.e(\"Target not defined.\")\n",
      "                    try:\n",
      "                        my_location = rospy.search_param(self.name)[0]\n",
      "                    except:\n",
      "                        self.logger.e(\"Cannot deduce target\")\n",
      "                        return\n",
      "            self._target = f'{my_location}/{self.name}'\n",
      "            self.logger.i(f\"Parameter target is {self._target}\")\n",
      "        else:\n",
      "            self._target = var\n",
      "            self.logger.i(f\"Parameter target is {self._target}\")\n",
      "    @property\n",
      "    def default(self):\n",
      "        if self._default == None:\n",
      "            self.default = None\n",
      "        return self._default\n",
      "    @default.setter\n",
      "    def default (self, var=None):\n",
      "        if var == None:\n",
      "            if self.value != None:\n",
      "                self._default = self.value\n",
      "            else:\n",
      "                try:\n",
      "                    self._default = self.get_parameter(self.target)\n",
      "                    self.logger.i(\"No default value given. Setting default to parameter.\")\n",
      "                except:\n",
      "                    self.logger.e(\"No default value given, and no global parameter set.\")\n",
      "        else:\n",
      "            #if self._value == None:\n",
      "            #    self._value = var\n",
      "            self._default = var\n",
      "    @property\n",
      "    def dynamic(self):\n",
      "        if self._dynamic == None:\n",
      "            self.dynamic = None\n",
      "        return self._dynamic\n",
      "    @dynamic.setter\n",
      "    def dynamic(self, var: bool = False):\n",
      "        self._dynamic = var\n",
      "    \n",
      "    def get_parameter(self, target):\n",
      "        if rospy.has_param(target):\n",
      "            self.logger.i(f'Parameter {self.name} found. Retreiving.')\n",
      "            return rospy.get_param(target)\n",
      "        else:\n",
      "            pass\n",
      "\n",
      "    def set_parameter(self, target, value):\n",
      "        rospy.set_param(target, value)\n",
      "        self.logger.i(f'Parameter {self.name} has been set to {self.value}.')\n",
      "        \n",
      "    def refresh_parameter(self):\n",
      "        if self.dynamic:\n",
      "            self.value = self.get_parameter(self.target)\n",
      "        else:\n",
      "            pass\n",
      "            \n",
      "    @property\n",
      "    def value(self):\n",
      "        return self._value\n",
      "    @value.setter\n",
      "    def value(self, var=None):\n",
      "        if var == None:\n",
      "            if self.dynamic:\n",
      "                if rospy.has_param(self.target):\n",
      "                    if self.get_parameter(self.target) != None:\n",
      "                        self._value = self.get_parameter(self.target)\n",
      "                        self.logger.i(f\"Dynamic retrieval. Using {self.name} is {self._value}.\")\n",
      "                    else:\n",
      "                        self._value = self.default\n",
      "                else:\n",
      "                    self._value = self.default\n",
      "            else:\n",
      "                self._value = self.default\n",
      "                self.logger.i(\"Dynamic retrieval turned off. Using Default.\")\n",
      "        else:\n",
      "            self._value = var\n",
      "\n"
     ]
    }
   ],
   "source": [
    "lines = inspect.getsource(Parameter)\n",
    "print(lines)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.8.10"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
